@@ -15,4 +15,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-backgroundFiller = (0,0,0)
\ No newline at end of file
+backgroundFiller = (0,0,0)
+shadowDefault = (128,128,128)
+shadowAgainstGrey = (0,0,0)
\ No newline at end of file
