@@ -48,6 +48,16 @@ class ScaledPoint(object):
     def __repr__(self):
         return "Scaled Point (%d, %d)" % self.val
 
+class ScaledScalar(object):
+    def __init__(self, val):
+        self.val = val
+        
+    def getVal(self, app):
+        return max(1, app.screenManager.scaleFactor * self.val)
+
+    def __repr__(self):
+        return "ScaledScalar (%d)" % (self.val,)
+
 class FullScreenAttachedPoint(object):
     def __init__(self, distance, attachedAt = 'topleft'):
         self.val = distance
@@ -88,6 +98,7 @@ class AttachedPoint(object):
             val = self.val.getSize(app)
         else:
             val = self.val
+
         return addPositions(pos, val)
 
 class ScaledSize(object):
@@ -98,6 +109,14 @@ class ScaledSize(object):
     def __repr__(self):
         return "Scaled Size (%d, %d)" % self.val
 
+class Size(object):
+    def __init__(self, x, y):
+        self.val = (x, y)
+    def getSize(self, app):
+        return self.val
+    def __repr__(self):
+        return "Size (%d, %d)" % self.val
+
 class FullScreenSize(object):
     def getSize(self, app):
         return app.screenManager.size
@@ -197,7 +216,7 @@ class RelativeLocation(Location):
         self.location = location
 
     def __repr__(self):
-        return "RelativeLocation: (%s plus %s)" % (repr(self.location), repr(self.size))
+        return 'RelativeLocation: (%s plus %s)' % (repr(self.location), repr(self.size))
         
     def apply(self, app, rect):
         self.location.apply(app, rect)
@@ -215,9 +234,9 @@ class Area(Location):
                     the point will be attached to
     '''
     def __init__(self, point, size, anchor=defaultAnchor):
-        self.point = point
+        Location.__init__(self, point, anchor)
         self.size = size
-        self.anchor = anchor
+
     def apply(self, app, rect):
         '''Moves and resizes this rect so that it covers this area.'''
         if hasattr(self.size, 'getSize'):
@@ -237,9 +256,14 @@ class Area(Location):
 
 class PygameEvent(object):
     def __init__(self, **kwargs):
+        self._repr = 'PygameEvent(%s)' % (', '.join('%s=%s' % a for a in
+                kwargs.iteritems()))
         for k, v in kwargs.iteritems():
             setattr(self, k, v)
 
+    def __repr__(self):
+        return self._repr
+
 def translateEvent(event, amount):
     result = PygameEvent(type=event.type, pos=(event.pos[0] - amount[0],
                                          event.pos[1] - amount[1]))
