@@ -29,6 +29,11 @@ def get(name, size):
             filename = getPath(fonts, name)
         else:
             filename = None
-        result = pygame.font.Font(filename, size)
+        try:
+            result = pygame.font.Font(filename, size)
+        except IOError:
+            print 'Font Cache: unable to load font %r' % (filename,)
+            result = pygame.font.Font(None, size)
+
         _cache[name, size] = result
         return result
