@@ -26,10 +26,10 @@ import random
 from twisted.internet import reactor
 
 from trosnoth.src.utils.utils import timeNow, new
-from trosnoth.src import modes
+from trosnoth.src.model import modes
 from trosnoth.src.utils import logging
-from trosnoth.src.universe_base import GameState
-from trosnoth.src.transaction import TransactionState
+from trosnoth.src.model.universe_base import GameState
+from trosnoth.src.model.transaction import TransactionState
 
 # TODO: Callbacks/callLaters should validate server-side before sending to
 # everyone; make sure the player hasn't left since calling.
@@ -45,7 +45,7 @@ class Team(object):
         self.currentTransaction = None
         self.numOrbsOwned = 0
         self.teamStars = 0
-        self.udpAddrs = {}
+        self.connIds = {}
         self.captain = None
         self.ready = False
 
@@ -61,28 +61,6 @@ class Team(object):
     def __repr__(self):
         return '%s team' % self.teamName
 
-    def teamString(self):
-        '''The teamString is sent to clients to notify them about this
-        team's current state.'''
-        
-        message = self.id
-        if self.captain:
-            message += 'T'
-            message += self.captain.id
-        else:
-            message += 'F'
-        if self.ready:
-            message += 'T'
-        else:
-            message += 'F'
-        message += struct.pack('I', self.numOrbsOwned)
-        message += struct.pack('B', len(self.udpAddrs))
-        if self.currentTransaction:
-            message += 'T'
-        else:
-            message += 'F'
-        return message
-
     def teamReady(self, initiator = None):
         if initiator == self.captain and not self.ready:
             self.ready = True
@@ -130,26 +108,23 @@ class Team(object):
     def addStar(self):
         self.teamStars += 1
 
-    def addAddress(self, pId, udpAddress):
-        self.udpAddrs[pId] = udpAddress
+    def addAddress(self, pId, connId):
+        self.connIds[pId] = connId
 
     def isCaptain(self, player):
         return player == self.captain
 
     def leave(self, player):
         '''Cleans up after the leaving player.'''
-        del self.udpAddrs[player.id]
+        del self.connIds[player.id]
         self.playerDie(player)
         player.currentZone.removePlayer(player)
         if self.captain == player:
             self.captain = None
             self.ready = False
 
-    def hasAddress(self, address):
-        return self.udpAddrs.values().__contains__(address)
-
-    def changeTeamName(self, name):
-        self.teamName = name
+    def hasAddress(self, connId):
+        return connId in self.connIds.values()
 
     @staticmethod
     def setOpposition(teamA, teamB):
@@ -475,17 +450,15 @@ class Universe(object):
     def setGameMode(self, mode):
         try:
             getattr(modes.GameMode(Shot, None, None), mode)()
-            self.gameMode = mode
-            print 'Server: set game mode to ' + mode
-            self.netServer.sendGameMode()
-            return True
         except AttributeError:
-            print "No such gameMode"
-            return False
-        except TypeError:
-            print "No such gameMode"
+            print 'No such gameMode'
             return False
 
+        self.gameMode = mode
+        print 'Server: set game mode to ' + mode
+        self.netServer.sendGameMode()
+        return True
+
     def timeLeft(self):
         if self.gameState != GameState.InProgress or self.gameTimeLimit == 0:
             return None
@@ -755,174 +728,6 @@ class BackwardInterfaceMapBlock(InterfaceMapBlock):
         d = 0.8 * abs(relPos[1] - relPos[0] * 3 / 4)
         return d
 
-class Obstacle(object):
-    '''Represents an obstacle which can't be passed by player or shot.'''
-    __slots__ = ['pt1', 'deltaPt']
-
-    def __init__(self, pt1, deltaPt):
-        self.pt1 = pt1
-        self.deltaPt = deltaPt
-
-    def collide(self, pt, deltaX, deltaY):
-        '''Returns how far a point-sized object at the specified position
-        could travel if it were trying to travel a displacement of
-        [deltaX, deltaY].
-        '''
-
-        ax, ay = self.pt1
-        bx, by = pt
-        dX1, dY1 = self.deltaPt
-
-        # Check if the lines are parallel.
-        denom = dX1*deltaY - deltaX*dY1
-        if denom <= 1e-10:
-            # We can go through it in this direction.
-            return deltaX, deltaY
-
-        # Calculate whether the line segments intersect.
-
-        s = (deltaX * (ay-by) - deltaY * (ax-bx)) / denom
-        # Take into account floating point error.
-        if s < -1e-10 or s > (1.+1e-10):
-            return deltaX, deltaY
-
-        t = (dX1 * (ay-by) - dY1 * (ax-bx)) / denom
-        if t < -1e-10 or t > (1.+1e-10):
-            return deltaX, deltaY
-
-        # Calculate the actual collision point.
-        x = bx + t*deltaX
-        y = by + t*deltaY
-
-        # Return the allowed change in position.
-        return x - pt[0], y - pt[1]
-
-    def finalPosition(self, pt, deltaX, deltaY):
-        '''Returns the final position of a point-sized object at the specified
-        position which is attempting to travel along the given displacement.
-        May assume that the object hits this obstacle on the way.
-        If the final position is simply the collision point, should return
-        None.'''
-        return None
-    
-class JumpableObstacle(Obstacle):
-    '''Represents an obstacle that is not a wall.'''
-
-    def hitByPlayer(self, player):
-        '''This obstacle has been hit by player.'''
-        player._jumpTime = 0
-        player.yVel = 0
-
-class GroundObstacle(JumpableObstacle):
-    '''Represents an obstacle that players are allowed to walk on.'''
-    drop = False
-    def walkTrajectory(self, distance):
-        '''Returns the displacement that a player would be going on this
-        surface if they would travel a horizontal distance of distance in
-        free space.'''
-        ratio = (distance+0.) / self.deltaPt[0]
-        return [ratio * self.deltaPt[i] for i in (0,1)]
-
-    def inBounds(self, pos):
-        '''Checks whether a given point is within the boundaries of this
-        obstacle. Should be called by a player object who was on this piece
-        of ground, and has just moved.'''
-        return self.pt1[0] <= pos[0] <= self.pt1[0] + self.deltaPt[0]
-
-class LedgeObstacle(GroundObstacle):
-    drop = True
-    
-class RoofObstacle(Obstacle):
-    def __init__(self, pt1, deltaPt):
-        super(RoofObstacle, self).__init__(pt1, deltaPt)
-        
-    def finalPosition(self, pt, deltaX, deltaY):
-        '''Returns the final position of a point-sized object at the specified
-        position trying to travel a displacement of [deltaX, deltaY], given
-        that it collides with this obstacle.'''
-        # If an object collides with a roof obstacle while falling, it still
-        #  falls.
-        
-        if deltaY > 0:
-            # We collided with the roof while falling.
-            # New position is where the roof is at the correct y-position.
-
-            # I added a quick fix to the bug that causes an infinite loop
-            # while players are touching a sloped roof, by adding 0.1 to
-            # the player's position, in the direction opposite travel. Have
-            # not been able to reproduce the bug since, however should be
-            # looked at some more. -Ashley (Smashery)
-            x = self.pt1[0] + (pt[1] + deltaY - self.pt1[1]) * \
-                 self.deltaPt[0] / (0. + self.deltaPt[1]) - \
-                 ((deltaX / abs(deltaX)) * 0.1)
-            y = pt[1] + deltaY
-            # Note: the above statement should only ever cause a division by
-            #  zero if someone has been silly enough to put a piece of
-            #  horizontal roof in upside down (impassable from above not below)
-            return x, y
-
-        # Normal case:
-        return None
-
-    def hitByPlayer(self, player):
-        # Stop any upward motion of the player.
-        player.yVel = max(player.yVel, 0)
-        player._jumpTime = 0
-
-class FillerRoofObstacle(RoofObstacle):
-    '''Represents an obstacle that would be used as an obstacleEdge;
-    its endpoint calculations are less precise (that being they
-    don't exist), but are able to be so due to their usage.'''
-    
-    def finalPosition(self, pt, deltaX, deltaY):
-        if self.deltaPt[0] == 0:
-            return pt[0], pt[1] + (deltaY/2)
-        else:
-            super(FillerRoofObstacle, self).finalPosition(pt, deltaX, deltaY)
-
-    def collide(self, pt, deltaX, deltaY):
-        '''Returns how far a point-sized object at the specified position
-        could travel if it were trying to travel a displacement of
-        [deltaX, deltaY].
-        '''
-        ax, ay = self.pt1
-        bx, by = pt
-        dX1, dY1 = self.deltaPt
-
-        # Check if the lines are parallel.
-        denom = dX1*deltaY - deltaX*dY1
-        if denom <= 0:
-            # We can go through it in this direction.
-            return deltaX, deltaY
-
-        # Calculate whether the line segments intersect.
-
-        s = (deltaX * (ay-by) - deltaY * (ax-bx)) / denom
-        # Take into account floating point error.
-        if s <= 0 or s >= 1:
-            return deltaX, deltaY
-
-        t = (dX1 * (ay-by) - dY1 * (ax-bx)) / denom
-        if t < -1e-10 or t > (1.+1e-10):
-            return deltaX, deltaY
-
-        # Calculate the actual collision point.
-        x = bx + t*deltaX
-        y = by + t*deltaY
-
-        # Return the allowed change in position.
-        return x - pt[0], y - pt[1]
-
-class VerticalWall(JumpableObstacle):
-    '''Represents a vertical wall that players can cling to and climb.'''
-    drop = True
-    
-    def __init__(self, pt1, deltaPt):
-        assert deltaPt[0] == 0
-
-        # Can drop off a vertical wall, so drop is always set to True
-        super(VerticalWall, self).__init__(pt1, deltaPt)
-
 class Zone(object):
     '''Represents information about a given zone and its state.'''
 
@@ -941,19 +746,22 @@ class Zone(object):
         if self.id == None:
             return 'Z---'
         return 'Z%3d' % self.id
-        
-    def tag(self, team, tagger):
-        '''This method should be called by netClient when the orb in this
-        zone is tagged. We assume that the tagging player is alive, that it
-        is not a Turret, and that the team does not already own the orb.'''
-        # Verify that the number of players within is in the team's favour
+
+    def canBeTaggedBy(self, tagger):
+        team = tagger.team
+        if self.orbOwner == team:
+            return False
+
         numTaggers = 0
         numDefenders = 0
-        if team:
+        # Create a temporary team reference to count
+        # members on each team
+        if team is not None:
             tempTeam = team
         else:
             tempTeam = self.universe.teams[0]
-            
+
+        # Count the players in this zone        
         for player in self.players:
             if player.turret:
                 # Turreted players do not count as a player for the purpose
@@ -964,24 +772,35 @@ class Zone(object):
             else:
                 numDefenders += 1
                     
-        if tagger and team:
+        if team is not None:
             # Zone has been tagged for a team:
             
             # Verify that the player is in this zone.
-            if not tagger.currentZone == self:
-                # We seem to think that the player is not within this zone.
-                # However, seeing as though the player says it has tagged this
-                # zone, it is safe to assume we can change his zone.
-                tagger.changeZone(self)
-                numTaggers += 1
+            if tagger.currentZone != self:
+                return False
 
 
-            if not (numTaggers > numDefenders or numTaggers > 4):
+            if not (numTaggers > numDefenders or numTaggers >= 4):
                 return False
         else:
             # Zone to be Rendered neutral
-            if not (numTaggers > 4 and numDefenders > 4):
+            if not (numTaggers >= 4 and numDefenders >= 4):
                 return False
+
+        # If an adjacent zone is also owned, we are able to tag
+        for zone in self.adjacentZones:
+            if zone.orbOwner == team:
+                return True
+        # Otherwise, we are unable to tag
+        return False     
+        
+    def tag(self, tagger):
+        '''This method should be called by netClient when the orb in this
+        zone is tagged. We assume that the tagging player is alive, that it
+        is not a Turret, and that the team does not already own the orb.'''
+        # Verify that the number of players within is in the team's favour
+        assert self.canBeTaggedBy(tagger)
+        team = tagger.team
         
         # Kill any turreted Player
         if self.turretedPlayer:
@@ -994,7 +813,7 @@ class Zone(object):
             team.orbGained()
 
             
-        if team == None:
+        if team is None:
             self.orbOwner = None
             self.zoneOwner = None
             print 'Zone %s neutralised' % (self.id,)
@@ -1080,6 +899,7 @@ class Player(object):
         self.turret = False
         self.shielded = False
         self.phaseshift = False
+        self.hasRicochet = False
         self.team = team
         self.universe = universe
         self.nick = nick
@@ -1168,7 +988,7 @@ class Shot(object):
     speed = 400       # pix/s
     lifetime = 1.2    # s
 
-    def __init__(self, id, team, pos, angle, player, turret):
+    def __init__(self, id, team, pos, angle, player, turret, ricochet):
 
         self.id = id
         self.team = team
@@ -1176,6 +996,7 @@ class Shot(object):
         self.angle = angle
         self.originatingPlayer = player
         self.turretShot = turret
+        self.ricochetShot = ricochet
         self.originatingPlayer.addShot(self)
         reactor.callLater(self.lifetime, self.shotDead)
 
