@@ -24,14 +24,19 @@ import pygame
 
 from trosnoth.src.utils.utils import new, timeNow
 from trosnoth.src.utils.logging import writeException
-from trosnoth.src import universe
+from trosnoth.src.model import universe
 from trosnoth.src.trosnothgui.ingame.minimap import MiniMap
 from trosnoth.src.trosnothgui.ingame import colours
 from trosnoth.src.gui.framework import framework, elements
 from trosnoth.src.trosnothgui.ingame.leaderboard import LeaderBoard
-from trosnoth.src.trosnothgui.ingame.statusBar import ZoneProgressBar, TimerBar
+from trosnoth.src.trosnothgui.ingame.statusBar import ZoneProgressBar
+from trosnoth.src.trosnothgui.ingame.gameTimer import GameTimer
 from trosnoth.src.trosnothgui.ingame.chatBox import ChatBox
-from trosnoth.src.universe_base import GameState
+from trosnoth.src.trosnothgui.ingame.universegui import UniverseGUI
+from trosnoth.src.model.universe_base import GameState
+
+from trosnoth.src.model.map import MapLayout
+from trosnoth.src.trosnothgui.ingame.sprites import PlayerSprite
 
 from trosnoth.src.gui.common import Location, FullScreenAttachedPoint, ScaledSize
 
@@ -86,7 +91,7 @@ class ViewManager(framework.CompoundElement):
     def setTarget(self, target):
         '''Makes the viewManager's target the specified value.'''
         self.target = target
-        if isinstance(self.target, universe.Player):
+        if isinstance(self.target, PlayerSprite):
             # Move directly to looking at player.     
             self._focus = target.pos
             self.reDraw()
@@ -99,7 +104,7 @@ class ViewManager(framework.CompoundElement):
         '''Returns the position of the current target.'''
         if self.target is None:
             return self._focus
-        if isinstance(self.target, universe.Player):
+        if isinstance(self.target, PlayerSprite):
             return self.target.pos
         return self.target
 
@@ -191,7 +196,7 @@ class ViewManager(framework.CompoundElement):
         self.backdrop.fill((64, 0, 64), rect)
         
         # Find which map blocks are within the bounds of that rect.
-        i, j = universe.MapLayout.getMapBlockIndices(self._focus[0] - self.sRect.center[0] + rect.left - self.sRect.left,
+        i, j = MapLayout.getMapBlockIndices(self._focus[0] - self.sRect.center[0] + rect.left - self.sRect.left,
                                                      self._focus[1] - self.sRect.center[1] + rect.top - self.sRect.top)
         
         y = i
@@ -241,14 +246,15 @@ class ViewManager(framework.CompoundElement):
         '''
         
         # Calculate where we should be looking at.
-        if isinstance(self.target, universe.Player):
+        if isinstance(self.target, PlayerSprite):
             # Take into account where the player's looking.
             plPos = self.target.pos
             plAngle = self.target.angleFacing
 
-            radius = self.target.distanceToTarget
-            if radius == None or radius > self.lengthFromPlayer:
-                radius = self.lengthFromPlayer
+            radius = self.target.ghostThrust * self.lengthFromPlayer
+            #distanceToTarget
+            #if radius == None or radius > self.lengthFromPlayer:
+            #    radius = self.lengthFromPlayer
                 
             targetPt = (plPos[0] + radius * sin(plAngle),
                         plPos[1] - radius * cos(plAngle))
@@ -263,9 +269,9 @@ class ViewManager(framework.CompoundElement):
             countdown, players = self.autoFocusInfo
 
             # First check for non-existent players.
-            for i in range(len(players)):
-                if players[-i-1] not in self.universe.players:
-                    players.pop(-i-1)
+            for p in players:
+                if p not in self.universe.players:
+                    players.remove(p)
 
             if len(self.universe.players) == 0:
                 # No players anywhere. No change.
@@ -280,8 +286,8 @@ class ViewManager(framework.CompoundElement):
                     if count > maxP:
                         maxP = count
                         curZone = z
-                self.autoFocusInfo[1] = list(curZone.players) + \
-                                        list(curZone.nonPlayers)
+                self.autoFocusInfo[1] = [self.universe.getPlayerSprite(p) for
+                    p in (list(curZone.players) + list(curZone.nonPlayers))]
 
                 targetPt = tuple(self._focus)
                 countdown = 10
@@ -311,6 +317,10 @@ class ViewManager(framework.CompoundElement):
                                 curZone = z
                         self.autoFocusInfo[1] = list(curZone.players) + \
                                                 list(curZone.nonPlayers)
+                        self.autoFocusInfo[1] = [
+                                self.universe.getPlayerSprite(p) for p in
+                                (list(curZone.players) +
+                                list(curZone.nonPlayers))]
                     countdown = 10
                 countdown -= 1
         
@@ -352,14 +362,18 @@ class ViewManager(framework.CompoundElement):
         self.drawBackdropRect(self.sRect)
 
 class PregameMessage(elements.TextElement):
-    def __init__(self, app):
+    def __init__(self, app, pos = None):
+        if pos is None:
+            pos = Location(FullScreenAttachedPoint((0, -150), 'center'), 'center')
         super(PregameMessage, self).__init__(
             app,
             'Game will begin when both captains signal ready.',
             app.screenManager.fonts.bigMenuFont,
-            Location(FullScreenAttachedPoint((0, -150), 'center'), 'center'),
-            colours.pregameMessageColour
+            pos,
+            colours.pregameMessageColour,
+            shadow = True
         )
+        self.setShadowColour((0,0,0))
 
 class GameViewer(framework.CompoundElement):
     '''The gameviewer comprises a viewmanager and a minimap, which can be
@@ -373,38 +387,39 @@ class GameViewer(framework.CompoundElement):
         super(GameViewer, self).__init__(app)
         self.interface = gameInterface
         self.world = world
+        self.worldgui = UniverseGUI(self.world)
         self.app = app
 
-        self.viewManager = ViewManager(self.app, world)
+        self.viewManager = ViewManager(self.app, self.worldgui)
         miniMapSize = (min(self.app.screenManager.scaledSize[0] / 100 * 35, 300),
                     min(self.app.screenManager.scaledSize[1] / 100 * 35, 200))
         self.miniMap = MiniMap(self.app, (self.app.screenManager.size[0] - miniMapSize[0] - 5, 5),\
                                        miniMapSize, 20, world, \
                                        self.viewManager)
 
-        xPos = int(self.app.screenManager.scaleFactor * -40)
-        yPos = self.miniMap.sRect.bottom + self.zoneBarHeight + 10
-        self.leaderboard = LeaderBoard(self.app, self.world, 50, Location(FullScreenAttachedPoint((xPos, yPos), 'topright'), 'topright'), 'right', 'top', self.app.screenManager.fonts.messageFont)
+        self.leaderboard = LeaderBoard(app, world, self)
         self.zoneBar = ZoneProgressBar(app, world, self)
-        self.timerBar = TimerBar(app, world)
+        self.timerBar = GameTimer(app, world)
         self.chatBox = ChatBox(self.app, self.world, self)
 
-        self.pregameMessage = PregameMessage(self.app)
+        xPos = -10
+        yPos = self.miniMap.sRect.bottom + self.zoneBarHeight - 5
+        messagePos = Location(FullScreenAttachedPoint((xPos, yPos), 'topright'), 'topright')
+        self.pregameMessage = PregameMessage(self.app, messagePos)
         if world.gameState == GameState.InProgress:
             self.pregameMessage.setText('')
 
-        self.elements = [self.viewManager, self.miniMap, self.pregameMessage]
+        self.elements = [self.viewManager, self.pregameMessage]
 
         self.minimapDisruptor = None
 
-        # Show zone bar and timer by default.
-        self.toggleZoneProgress()
-        self.toggleTimer()
+        self.toggleInterface()
+        self.toggleLeaderBoard()
                 
     def setTarget(self, target):
         'Target should be a player, a point, or None.'
         self.viewManager.setTarget(target)
-        if isinstance(target, universe.Player):
+        if isinstance(target, PlayerSprite):
             if self.minimapDisruptor:
                 if target.team != self.minimapDisruptor.team:
                     self.miniMap.disrupted()
@@ -412,12 +427,12 @@ class GameViewer(framework.CompoundElement):
                     self.miniMap.endDisruption()
         
     def tick(self, deltaT):
-        self.world.tick()
+        self.worldgui.tick(deltaT)
         super(GameViewer, self).tick(deltaT)
 
     def minimapDisruption(self, player):
         self.minimapDisruptor = player
-        if isinstance(self.viewManager.target, universe.Player):
+        if isinstance(self.viewManager.target, PlayerSprite):
             if player.team != self.viewManager.target.team:
                 self.miniMap.disrupted()
 
@@ -425,25 +440,23 @@ class GameViewer(framework.CompoundElement):
         self.minimapDisruptor = None
         self.miniMap.endDisruption()
 
+    def toggleInterface(self):
+        if self.miniMap in self.elements:
+            self.elements.remove(self.zoneBar)
+            self.elements.remove(self.timerBar)
+            self.elements.remove(self.miniMap)
+        else:
+            self.elements.append(self.zoneBar)
+            self.elements.append(self.timerBar)
+            self.elements.append(self.miniMap)
+            self.timerBar.syncTimer()
+
     def toggleLeaderBoard(self):
         if self.leaderboard in self.elements:
             self.elements.remove(self.leaderboard)
         else:
             self.elements.append(self.leaderboard)
 
-    def toggleZoneProgress(self):
-        if self.zoneBar in self.elements:
-            self.elements.remove(self.zoneBar)
-        else:
-            self.elements.append(self.zoneBar)
-
-    def toggleTimer(self):
-        if self.timerBar in self.elements:
-            self.elements.remove(self.timerBar)
-        else:
-            self.elements.append(self.timerBar)
-            self.timerBar.syncTimer()
-
     def openChat(self, player):
         self.chatBox.reset()
         self.chatBox.setPlayer(player)
