@@ -25,72 +25,64 @@ from twisted.internet import protocol, reactor, task
 from twisted.internet.error import CannotListenError
 from twisted.internet.protocol import DatagramProtocol
 
-import trosnoth.src.trosnothgui.defines as defines
-import trosnoth.src.gui.framework.framework as framework
+from trosnoth.src.utils.utils import timeNow
+from trosnoth.src.utils import logging
+from trosnoth.src.trosnothgui import defines
+from trosnoth.src.gui.framework import framework
+from trosnoth.src.gui.framework import prompt
+from trosnoth.src.gui.framework.hint import Hint
 import trosnoth.src.gui.framework.prompt as prompt
 from trosnoth.src.gui.framework.elements import PictureElement, TextElement, TextButton
 from trosnoth.src.gui.framework.tab import Tab
 from trosnoth.src.gui.framework.checkbox import CheckBox
 from trosnoth.src.gui.framework.listbox import ListBox
 from trosnoth.src.gui.framework.dialogbox import DialogBox, DialogResult, DialogBoxAttachedPoint
-import trosnoth.data.startupMenu as startupMenu
+from trosnoth.data import startupMenu
 from trosnoth.src.trosnothgui.pregame import colours
 from trosnoth.src.gui.fonts.font import ScaledFont
 
-from trosnoth.src.networkDefines import *
-
 from trosnoth.src.gui.common import *
 
 from trosnoth.src.trosnothgui.pregame.imageRadioButton import ImageRadioButton, RadioButtonGroup
 from trosnoth.src.trosnothgui.pregame.promptImageRadioButton import PromptImageRadioButton
 
-import trosnoth.data.startupMenu as startupMenu
 from trosnoth.data import getPath
 
-
-def localServerRunning():
-    port = None
-
-    # Try binding port 6789
-    try:
-        port = reactor.listenTCP(6789, TestFactory())
-    except CannotListenError:
-        pass
-
-    # If it was successful, there is no server: stop listening
-    if port:
-        port.stopListening()
-        return False
-    else:
-        return True
-
-class TestProtocol(protocol.Protocol):
-    pass
-class TestFactory(protocol.ServerFactory):
-    protocol = TestProtocol
+#TODO: remove from networkDefines
+from trosnoth.src.network.networkDefines import maxTeamNameLength
 
 class ServerSetupTab(Tab):
     def __init__(self, app, tabContainer, startupInterface):
-        super(ServerSetupTab, self).__init__(app, "Start a Game")
+        super(ServerSetupTab, self).__init__(app, 'Start a Game')
+        self.tabContainer = tabContainer
         self.toCreate = ServerDoesntExistMenu(app, tabContainer, startupInterface, self)
         self.created = ServerAlreadyExistsMenu(app, tabContainer, startupInterface, self)
-        self.tabContainer = tabContainer
         self.setAppropriateElements()
 
         # Refresh the elements list every now and then in case server has shut down.
-        task.LoopingCall(self.setAppropriateElements).start(5, False)
+        self._lastUpdate = timeNow()
 
     def setAppropriateElements(self):
-        if localServerRunning():
+        if self.app.server is not None:
             self.elements = [self.created]
-            self.tabContainer.renameTab("Game Options", self)
+            self.tabContainer.renameTab('Game Options', self)
         else:
             self.elements = [self.toCreate]
-            self.tabContainer.renameTab("Start a Game", self)
+            self.tabContainer.renameTab('Start a Game', self)
 
     def setName(self, name):
         self.toCreate.setName(name)        
 
+    def draw(self, screen):
+        # Automatically update every half second, but only lazily in case it's
+        # not visible.
+        time = timeNow()
+        if self._lastUpdate < time - 0.5:
+            self.setAppropriateElements()
+            self._lastUpdate = time
+        
+        super(ServerSetupTab, self).draw(screen)
+
 class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
     def __init__(self, app, tabContainer, startupInterface, parent):
         super(ServerDoesntExistMenu, self).__init__(app)
@@ -112,23 +104,23 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         imageLong  = SizedImage(getPath(startupMenu, 'long.png'), imageSize, (0,0,0))
         imageLarge = SizedImage(getPath(startupMenu, 'large.png'), imageSize, (0,0,0))
 
-        largeRadio = ImageRadioButton(app, "Standard", ScaledArea(85,370,200,150), imageLarge)
+        largeRadio = ImageRadioButton(app, 'Standard', ScaledArea(85,370,200,150), imageLarge)
         largeRadio.value = (3,2)
         self.radio.add(largeRadio)
 
-        smallRadio = ImageRadioButton(app, "Small", ScaledArea(305, 370,200,150), imageSmall)
+        smallRadio = ImageRadioButton(app, 'Small', ScaledArea(305, 370,200,150), imageSmall)
         smallRadio.value = (1,1)
         self.radio.add(smallRadio)
 
-        longRadio = ImageRadioButton(app, "Wide", ScaledArea(85,540,200,150), imageLong)
+        longRadio = ImageRadioButton(app, 'Wide', ScaledArea(85,540,200,150), imageLong)
         longRadio.value = (5,1)
         self.radio.add(longRadio)
 
-        customRadio = PromptImageRadioButton(app, "Custom", ScaledArea(305,540,200,150))
+        customRadio = PromptImageRadioButton(app, 'Custom', ScaledArea(305,540,200,150))
         self.radio.add(customRadio)
 
         self.radio.selected(largeRadio)
-        self.radio.onSelectionChanged.addListener(self.setPlayerNumber)   
+        self.radio.onSelectionChanged.addListener(self.setSelectionNumber)   
 
 
         self.text = [
@@ -139,13 +131,13 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                                  ScaledLocation(600, 220),
                                  colours.headingColour),
                      TextElement(self.app, 'players per side', app.screenManager.fonts.smallNoteFont,
-                                 ScaledLocation(720, 310),
+                                 ScaledLocation(720, 295),
                                  colours.headingColour),
                      TextElement(self.app, 'Time Limit:', font,
                                  ScaledLocation(600, 360), 
                                  colours.headingColour),
                      TextElement(self.app, 'minutes', app.screenManager.fonts.smallNoteFont,
-                                 ScaledLocation(720, 450), 
+                                 ScaledLocation(720, 435), 
                                  colours.headingColour),
                      ]
 
@@ -157,28 +149,29 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                                       font = font)
         self.serverInput.onClick.addListener(self.setFocus)
         self.serverInput.onTab.addListener(self.tabNext)
-        self.serverInput.onEnter.addListener(lambda sender: self.startServer())
+        self.serverInput.onEnter.addListener(self.startServer)
         
-        self.maxPlayersInput = prompt.InputBox(self.app, ScaledArea(600, 290, 100, 60),
-                                           initValue = '16',
+        self.maxPlayersInput = prompt.InputBox(self.app, ScaledArea(600, 275, 100, 60),
+                                           initValue = '10',
                                            font = font, maxLength = 3,
                                            validator = prompt.intValidator)
         self.maxPlayersInput.onClick.addListener(self.setFocus)
         self.maxPlayersInput.onTab.addListener(self.tabNext)
-        self.maxPlayersInput.onEnter.addListener(lambda sender: self.startServer())
+        self.maxPlayersInput.onEnter.addListener(self.startServer)
 
-        self.gameDurationInput = prompt.InputBox(self.app, ScaledArea(600, 430, 100, 60),
+        self.gameDurationInput = prompt.InputBox(self.app, ScaledArea(600, 415, 100, 60),
+                                           initValue = '45',
                                            font = font, maxLength = 3,
                                            validator = prompt.intValidator)
         self.gameDurationInput.onClick.addListener(self.setFocus)
         self.gameDurationInput.onTab.addListener(self.tabNext)
-        self.gameDurationInput.onEnter.addListener(lambda sender: self.startServer())
+        self.gameDurationInput.onEnter.addListener(self.startServer)
 
-        self.noLimitBox = CheckBox(self.app, ScaledLocation(830, 385),
+        self.noLimitBox = CheckBox(self.app, ScaledLocation(830, 375),
                                            text = 'No Limit',
                                            font = app.screenManager.fonts.smallNoteFont,
                                            colour = colours.mainMenuColour,
-                                           initValue = True,
+                                           initValue = False,
                                            style='circle')
         self.noLimitBox.onValueChanged.addListener(lambda sender: self.enableGameDuration(not sender.value))
 
@@ -189,34 +182,56 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                                            initValue = True,
                                            style='circle')
 
-        self.startButton = TextButton(self.app, ScaledLocation(600, 580), 'start', app.screenManager.fonts.hugeMenuFont, colours.startButton, colours.white)
-        self.startButton.onClick.addListener(lambda sender: self.startServer())
+        self.invisibleGameBox = CheckBox(self.app, ScaledLocation(800, 520),
+                                           text = 'Invisible',
+                                           font = app.screenManager.fonts.smallNoteFont,
+                                           colour = colours.mainMenuColour,
+                                           initValue = False,
+                                           style='circle')
+
+        igHint = Hint(app, 'Others will need to know your IP address to '
+                'connect.', self.invisibleGameBox,
+                font=app.screenManager.fonts.smallNoteFont)
+
+
+        self.startButton = TextButton(self.app, ScaledLocation(600, 610),
+                'start', app.screenManager.fonts.hugeMenuFont,
+                colours.startButton, colours.white)
+        self.startButton.onClick.addListener(self.startServer)
         
         self.input = [self.serverInput, self.maxPlayersInput, self.gameDurationInput]
-        self.enableGameDuration(False)
+        self.enableGameDuration(True)
 
         self.elements = self.text + self.input + \
                         [self.recordReplayBox,
+                         self.invisibleGameBox,
                          self.noLimitBox,
                          self.invalidInputText,
                          self.startButton,
-                         self.radio
+                         self.radio,
+                         igHint
                         ]
         self.setFocus(self.serverInput)
         self.lastSelection = -1
-        self.setPlayerNumber(0)   
+        self.setSelectionNumber(0)   
 
-    def setPlayerNumber(self, index):
-        playerSizes = [8, 3, 5]
+    def setSelectionNumber(self, index):
+        playerSizes = [10, 4, 6]
+        timeLimits = [45, 10, 20]
         chosenNum = int(self.maxPlayersInput.getValue())
         if (self.lastSelection == -1 or self.lastSelection == 3 or playerSizes[self.lastSelection] == chosenNum) and index != 3:
             self.maxPlayersInput.setValue(str(playerSizes[index]))
+        if not self.noLimitBox.value:
+            chosenLimit = int(self.gameDurationInput.getValue())
+            if (self.lastSelection == -1 or self.lastSelection == 3 or timeLimits[self.lastSelection] == chosenLimit) and index != 3:
+                self.gameDurationInput.setValue(str(timeLimits[index]))
+            
         if index != 3 and index != -1:
             self.lastSelection = index
         
 
     def setName(self, name):
-        #if self.serverInput.getValue() == "":
+        #if self.serverInput.getValue() == '':
             self.serverInput.setValue("%s's game" % (name,))
 
     def setTabOrder(self):
@@ -231,7 +246,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         hadFocus = self.gameDurationInput.hasFocus
         super(ServerDoesntExistMenu, self).setFocus(object)
         # If the game duration box had and now does not have focus
-        if hadFocus and not self.gameDurationInput.hasFocus and self.gameDurationInput.value == "":
+        if hadFocus and not self.gameDurationInput.hasFocus and self.gameDurationInput.value == '':
             self.enableGameDuration(False)
             self.noLimitBox.setValue(True)
 
@@ -247,11 +262,11 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         self.setTabOrder()
             
 
-    def startServer(self):
+    def startServer(self, sender):
         values = self.getValues()
-        if values:
-            self.incorrectInput("")
-            self.startupInterface.startServer(*values)
+        if values is not None:
+            self.incorrectInput('')
+            self.app.startServer(*values)
             self.parent.setAppropriateElements()
             # Select the LAN tab
             self.tabContainer._tabSelected(0)
@@ -268,12 +283,15 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         self.serverInput.setBackColour(colours.white)
 
     def getValues(self):
-        '''Returns the inputted values in the order:
+        '''
+        Returns the inputted values in the order:
         1. halfMapWidth
         2. mapHeight
         3. maxPlayers
         4. Game Duration
-        5. Record Replay?'''
+        5. Record Replay?
+        6. Invisible game?
+        '''
         # TODO: move these to a central location for server and interface to read
         numPlayers = (2, 128)
 
@@ -282,10 +300,10 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         invalid = False
         
         height = width = maxPlayers = duration = name = None
-        if self.serverInput.value == '':     # TODO: ensure there isn't one of that name already
+        if self.serverInput.value == '':
             if not invalid:
                 invalid = True
-                self.incorrectInput("Please give this game a name")
+                self.incorrectInput('Please give this game a name')
             self.serverInput.setBackColour(colours.invalidDataColour)
         else:
             name = self.serverInput.value
@@ -294,7 +312,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
             maxPlayers = self.getInt(self.maxPlayersInput.value)
         else:
             if not invalid:
-                self.incorrectInput("Must have between %d and %d players per team" %
+                self.incorrectInput('Must have between %d and %d players per team' %
                                     (numPlayers[0], numPlayers[1]))
                 invalid = True
             self.maxPlayersInput.setBackColour(colours.invalidDataColour)
@@ -308,23 +326,25 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                 self.gameDurationInput.setBackColour(colours.invalidDataColour)
                 if not invalid:
                     invalid = True
-                    self.incorrectInput("Please set a duration for this game") 
+                    self.incorrectInput('Please set a duration for this game') 
                 
         radioChoice = self.radio.getSelectedValue()
         if radioChoice is None:
             if not invalid:
                 invalid = True
-                self.incorrectInput("Please choose a map size")
+                self.incorrectInput('Please choose a map size')
 
         if invalid:
             return None
 
                     
         recordReplay = self.recordReplayBox.value
+        invisible = self.invisibleGameBox.value
         width, height = radioChoice
 
         
-        return [name, width, height, maxPlayers, duration, recordReplay]
+        return [name, width, height, maxPlayers, duration, recordReplay,
+                invisible]
 
     def getInt(self, value):
         if value == '':
@@ -361,6 +381,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         self.resetTimer = task.LoopingCall(self.resetCountdownFunction)
 
         self.delayedCall = None
+        self._lastStatusUpdate = timeNow()
 
         font = self.app.screenManager.fonts.bigMenuFont
 
@@ -389,7 +410,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         self.blueTeamInput = prompt.InputBox(self.app, ScaledArea(350, 320, 270, 60),
                                              initValue = self.blueTeamName,
                                              font = font, colour = colours.blueTeam,
-                                             maxLength = 10)
+                                             maxLength = maxTeamNameLength)
         self.blueTeamInput.onClick.addListener(self.setFocus)
         self.blueTeamInput.onEdit.addListener(lambda sender: self.inputEdit())
         self.blueTeamInput.onTab.addListener(self.tabNext)
@@ -398,7 +419,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         self.redTeamInput = prompt.InputBox(self.app, ScaledArea(655, 320, 270, 60),
                                             initValue = self.redTeamName,
                                             font = font, colour = colours.redTeam,
-                                            maxLength = 10)
+                                            maxLength = maxTeamNameLength)
         self.redTeamInput.onClick.addListener(self.setFocus)
         self.redTeamInput.onEdit.addListener(lambda sender: self.inputEdit())
         self.redTeamInput.onTab.addListener(self.tabNext)
@@ -438,58 +459,44 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         # By your powers combined, I am Captain Planet!
         self.elements = self.staticText + self.dynamicText + self.input + self.buttons
 
-        try:
-            # Initiate the StatusProtocol. Listen on default response port.
-            self.statusProtocol = StatusProtocol(self, defaultServerAdminResponsePort)
-
-            # Update the game status every second.
-            self.updateStatusLoop = task.LoopingCall(self.updateStatus)
-            self.updateStatusLoop.start(1.0, True)
-        except CannotListenError:
-            self.elements = self.staticText
-
-            self.elements = [TextElement(app, "There is a game running on this computer,", app.screenManager.fonts.bigMenuFont,
-                                         ScaledLocation(512, 290, 'center'), colours.noGamesColour),
-                             TextElement(app, "however the Game Options are already open", app.screenManager.fonts.bigMenuFont,
-                                         ScaledLocation(512, 350, 'center'), colours.noGamesColour),
-                             TextElement(app, "in another copy of Trosnoth.", app.screenManager.fonts.bigMenuFont,
-                                         ScaledLocation(512, 410, 'center'), colours.noGamesColour),
-                             TextElement(app, "Please use the first Trosnoth you opened or", app.screenManager.fonts.bigMenuFont,
-                                         ScaledLocation(512, 480, 'center'), colours.invalidDataColour),
-                             TextElement(app, "close all copies of Trosnoth and try again.", app.screenManager.fonts.bigMenuFont,
-                                         ScaledLocation(512, 540, 'center'), colours.invalidDataColour)]
-
+        self.updateStatus()
        
-    # This has the same effect as using the server admin tool,
-    # except it is hardcoded to port 6790
-    def sendCommand(self, command):
-        s = socket(AF_INET, SOCK_DGRAM)
-        n = s.sendto(command, ('127.0.0.1', defaultServerAdminPort))
-
-    def updateStatus(self):
-        self.sendCommand("GameStatus")
-
     def saveSettings(self):
-        nameChange = False
-        modeChange = False
-        if self.blueTeamInput.getValue().strip() != self.blueTeamName:
-            nameChange = True
-            self.sendCommand('TeamName0=' + self.blueTeamInput.getValue().strip())
-        if self.redTeamInput.getValue().strip() != self.redTeamName:
-            nameChange = True
-            self.sendCommand('TeamName1=' + self.redTeamInput.getValue().strip())
-        if self.gameModeInput.getValue().strip() != self.gameMode:
-            modeChange = True
-            self.sendCommand('GameMode=' + self.gameModeInput.getValue().strip())
-        if not modeChange:
-            if nameChange:
-                self.stopCountdown('Settings have been saved.')
+        cmd = self.getCmdInterface()
+        if cmd is None:
+            self.assistanceText.setText('Game is not running!')
+            self.assistanceText.setColour(colours.invalidDataColour)
+            self.inputEdit()
+            self.delayedErase(self.assistanceText)
+            return
+
+        blueName = self.blueTeamInput.getValue().strip()
+        if blueName != self.blueTeamName:
+            cmd.setTeamName(0, blueName)
+
+        redName = self.redTeamInput.getValue().strip() 
+        if redName != self.redTeamName:
+            cmd.setTeamName(1, redName)
+
+        gameMode = self.gameModeInput.getValue().strip()
+        if gameMode != self.gameMode:
+            success = cmd.setGameMode(gameMode)
         else:
-            self.stopCountdown('Waiting for response...')
-        self.assistanceText.setColour(colours.startButton)
+            success = True
+
+        if not success:
+            self.assistanceText.setText('Invalid game mode!')
+            self.assistanceText.setColour(colours.invalidDataColour)
+            self.inputEdit()
+        else:
+            self.stopCountdown('Settings have been saved.')
+            self.assistanceText.setColour(colours.startButton)
+        self.delayedErase(self.assistanceText)
 
     def startGame(self):
-        self.sendCommand('StartGame')
+        cmd = self.getCmdInterface()
+        if cmd is not None:
+            cmd.startGame()
         
     def shutdownServer(self):
         self.shutdownConfirmationBox = ShutdownDialog(self.app)
@@ -498,8 +505,9 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
 
     def _shutdownDlgClose(self):
         if self.shutdownConfirmationBox.result == DialogResult.OK:
-            self.sendCommand('Shutdown')
-            reactor.callLater(0.1, self.parent.setAppropriateElements)
+            cmd = self.getCmdInterface()
+            if cmd is not None:
+                cmd.shutdown()
 
     # The following three functions are all related: if a user edits the team names or gamde mode
     # but doesn't save them, they will get a certain amount of time before the inputs are reset to
@@ -516,13 +524,23 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
             self.stopCountdown('')
 
     def resetCountdownFunction(self):
+        try:
+            self._resetCountdownFunction()
+        except:
+            logging.writeException()
+
+    def _resetCountdownFunction(self):
         self.resetCountdown -= 1
         if self.resetCountdown <= 0:
-            self.stopCountdown("Settings reset due to inactivity")
+            self.stopCountdown('Settings reset due to inactivity')
             self.populateInputs()
             self.delayedErase(self.assistanceText)
         elif self.resetCountdown <= 5 or self.resetCountdown == 10 or self.resetCountdown == 20:
-            self.assistanceText.setText("Settings will reset in %d seconds" % self.resetCountdown)
+            if self.resetCountdown == 1:
+                _noun = "second"
+            else:
+                _noun = "seconds"
+            self.assistanceText.setText('Settings will reset in %d %s' % (self.resetCountdown, _noun))
             self.assistanceText.setColour(colours.startButton)
 
     def stopCountdown(self, message):
@@ -547,32 +565,97 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
     def clearText(self, element):
         element.setText('')
 
+    def getCmdInterface(self):
+        if self.app.server is not None:
+            return self.app.server.cmdInterface
+        
+        self.parent.setAppropriateElements()
+        return None
+
+    def updateStatus(self):
+        status = self.gameStatusText
+        cmd = self.getCmdInterface()
+        if cmd is None:
+            return
+        
+        currentGameState = cmd.getGameState()
+
+        if currentGameState != 'P':
+            self.startGameButton.setText('')
+
+        if currentGameState == 'P':
+            status.setText('Not yet started')
+            status.setColour(colours.inactive)
+            self.startGameButton.setText('force begin')
+        elif currentGameState == 'I':
+            blueScore, redScore = cmd.getOrbCounts()
+
+            if blueScore > redScore:
+                status.setText('In progress (%d-%d to %s)' %
+                        (blueScore, redScore, self.blueTeamName))
+                status.setColour(colours.blueTeam)
+            elif redScore > blueScore:
+                status.setText('In progress (%d-%d to %s)' %
+                        (redScore, blueScore, self.redTeamName))
+                status.setColour(colours.redTeam)
+            else:
+                status.setText('In progress (%d-%d tie)' %
+                        (redScore, blueScore))
+                status.setColour(colours.bothTeams)
+        elif currentGameState in ('D', 'B', 'R'):
+            if currentGameState == 'B':
+                status.setText('Game over - %s wins!' % self.blueTeamName)
+                status.setColour(colours.blueTeam)
+            elif currentGameState == 'R':
+                status.setText('Game over - %s wins!' % self.redTeamName)
+                status.setColour(colours.redTeam)
+            else:
+                status.setText('Game over - it was a draw!')
+                status.setColour(colours.bothTeams)
+            self.saveButton.setText('')
+        else:
+            self.gameStatusText.setText('Unknown')
+            self.gameStatusText.setColour(colours.inactive)
+
+        self.blueTeamName, self.redTeamName = cmd.getTeamNames()
+        self.gameMode = cmd.getGameMode()
+        if not self.inputsEdited:
+            self.populateInputs()
+
+    def draw(self, screen):
+        t = timeNow()
+        if t > self._lastStatusUpdate + 0.5:
+            self._lastStatusUpdate = t
+            self.updateStatus()
+
+        super(ServerAlreadyExistsMenu, self).draw(screen)
+
 class ShutdownDialog(DialogBox):
     '''Defines the elements and layout thereof of the server shutdown confirmation dialog box.'''
 
     def __init__(self, app):
-        super(ShutdownDialog, self).__init__(app, ScaledSize(450, 340), "Server Shutdown")
+        super(ShutdownDialog, self).__init__(app, ScaledSize(450, 340), 'Server Shutdown')
 
         # Warning message
-        text = [TextElement(app, "Are you sure you want", app.screenManager.fonts.mediumMenuFont,
+        text = [TextElement(app, 'Are you sure you want', app.screenManager.fonts.mediumMenuFont,
                                 Location(DialogBoxAttachedPoint(self, ScaledSize(0, 10), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, "to shutdown the server?", app.screenManager.fonts.mediumMenuFont,
+                TextElement(app, 'to shutdown the server?', app.screenManager.fonts.mediumMenuFont,
                                 Location(DialogBoxAttachedPoint(self, ScaledSize(0, 55), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, "Doing so will immediately", app.screenManager.fonts.mediumMenuFont,
+                TextElement(app, 'Doing so will immediately', app.screenManager.fonts.mediumMenuFont,
                                 Location(DialogBoxAttachedPoint(self, ScaledSize(0,130), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, "end the game and", app.screenManager.fonts.mediumMenuFont,
+                TextElement(app, 'end the game and', app.screenManager.fonts.mediumMenuFont,
                                 Location(DialogBoxAttachedPoint(self, ScaledSize(0,175), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, "disconnect all users!", app.screenManager.fonts.mediumMenuFont,
+                TextElement(app, 'disconnect all users!', app.screenManager.fonts.mediumMenuFont,
                                 Location(DialogBoxAttachedPoint(self, ScaledSize(0,220), 'midtop'), 'midtop'), colours.invalidDataColour)]
         
-        font = ScaledFont("KLEPTOCR.TTF", 50)
+        font = ScaledFont('KLEPTOCR.TTF', 50)
 
         # OK Button
-        okButton = TextButton(app,  Location(DialogBoxAttachedPoint(self, ScaledSize(-100,280), 'midtop'), 'midtop'), "Confirm", font, colours.dialogButtonColour, colours.radioMouseover)
+        okButton = TextButton(app,  Location(DialogBoxAttachedPoint(self, ScaledSize(-100,280), 'midtop'), 'midtop'), 'Confirm', font, colours.dialogButtonColour, colours.radioMouseover)
         okButton.onClick.addListener(lambda sender: self.ok())
 
         # Cancel Button
-        cancelButton = TextButton(app,  Location(DialogBoxAttachedPoint(self, ScaledSize(100,280), 'midtop'), 'midtop'), "Cancel", font, colours.dialogButtonColour, colours.radioMouseover)
+        cancelButton = TextButton(app,  Location(DialogBoxAttachedPoint(self, ScaledSize(100,280), 'midtop'), 'midtop'), 'Cancel', font, colours.dialogButtonColour, colours.radioMouseover)
         cancelButton.onClick.addListener(lambda sender: self.cancel())
 
         # Add elements to screen
@@ -587,80 +670,3 @@ class ShutdownDialog(DialogBox):
         self.result = DialogResult.Cancel
         self.Close()
 
-class StatusProtocol(DatagramProtocol):
-    '''This protocol is used to receive game status information from the server.'''
-
-    def __init__(self, menu, port):
-        self.menu = menu
-
-        # Ask the system for a UDP port.
-        self.port = reactor.listenUDP(port, self)
-
-    def datagramReceived(self, datagram, address):
-
-        status = self.menu.gameStatusText
-        
-        if address[0] != '127.0.0.1':
-            return
-
-        if datagram.startswith('GameStatus:'):
-            # GameStatus:I7.7
-            currentGameState = datagram[11]
-            if currentGameState != 'P':
-                self.menu.startGameButton.setText('')
-
-            if currentGameState == 'P':
-                status.setText('Not yet started')
-                status.setColour(colours.inactive)
-                self.menu.startGameButton.setText('force begin')
-            elif currentGameState == 'I':
-                decimalPos = datagram.find(".")
-                blueScore = int(datagram[12:decimalPos])
-                redScore = int(datagram[decimalPos+1:])
-
-                if blueScore > redScore:
-                    status.setText('In progress (%d-%d to %s)' %
-                            (blueScore, redScore, self.menu.blueTeamName))
-                    status.setColour(colours.blueTeam)
-                elif redScore > blueScore:
-                    status.setText('In progress (%d-%d to %s)' %
-                            (redScore, blueScore, self.menu.redTeamName))
-                    status.setColour(colours.redTeam)
-                else:
-                    status.setText('In progress (%d-%d tie)' %
-                            (redScore, blueScore))
-                    status.setColour(colours.bothTeams)
-            elif currentGameState in ('D', 'B', 'R'):
-                if currentGameState == 'B':
-                    status.setText('Game over - %s wins!' % self.menu.blueTeamName)
-                    status.setColour(colours.blueTeam)
-                elif currentGameState == 'R':
-                    status.setText('Game over - %s wins!' % self.menu.redTeamName)
-                    status.setColour(colours.redTeam)
-                else:
-                    status.setText('Game over - it was a draw!')
-                    status.setColour(colours.bothTeams)
-                self.menu.saveButton.setText('')
-            else:
-                self.menu.gameStatusText.setText('Unknown')
-                self.menu.gameStatusText.setColour(colours.inactive)
-
-        if datagram.startswith('TeamNames:'):
-            # TeamNames:Team 1    Team 2    GameMode
-            #           12345678901234567890...
-            self.menu.blueTeamName = datagram[10:20].strip()
-            self.menu.redTeamName = datagram[20:30].strip()
-            self.menu.gameMode = datagram[30:].strip()
-
-            if not self.menu.inputsEdited:
-                self.menu.populateInputs()
-
-        if datagram.startswith('Result:'):
-            if datagram[7:] == "InvalidGameMode":
-                self.menu.assistanceText.setText("Invalid game mode!")
-                self.menu.assistanceText.setColour(colours.invalidDataColour)
-                self.menu.inputEdit()
-            elif datagram[7:] == "Success":
-                self.menu.stopCountdown('Settings have been saved.')
-                self.menu.assistanceText.setColour(colours.startButton)
-            self.menu.delayedErase(self.menu.assistanceText)
